#include <Audio.h>
#include <Wire.h>
#include <SPI.h>

#include <SerialFlash.h>

// GUItool: begin automatically generated code
AudioSynthWaveformSine   sine1;          //xy=57,371
AudioSynthNoisePink      pink1;          //xy=57,413
AudioSynthWaveform       waveform1;      //xy=71,313
AudioMixer4              mixer1;         //xy=221,374
AudioEffectEnvelope      envelope1;      //xy=371,313
AudioFilterStateVariable filter1;        //xy=569,311
AudioOutputI2S           i2s1;           //xy=795,344
AudioConnection          patchCord1(sine1, 0, mixer1, 1);
AudioConnection          patchCord2(pink1, 0, mixer1, 2);
AudioConnection          patchCord3(waveform1, 0, mixer1, 0);
AudioConnection          patchCord4(mixer1, envelope1);
AudioConnection          patchCord5(envelope1, 0, filter1, 0);
AudioConnection          patchCord6(filter1, 0, i2s1, 0);
AudioConnection          patchCord7(filter1, 0, i2s1, 1);

AudioControlSGTL5000     sgtl5000_1;     //xy=251,102
// GUItool: end automatically generated code



int led_pin1 = 3;    // the number of the first LED pin
int ledState1 = LOW;             // ledState used to set the first LED
long previousMillis1 = 0;        // will store last time LED 1 was updated
long rate1;           // first led rate

int led_pin2 = 4;    // the number of the first LED pin
int ledState2 = LOW;             // ledState used to set the first LED
long previousMillis2 = 0;        // will store last time LED 1 was updated
long rate2;           // first led rate

long currentMillis;

int pot1_value;
int pot2_value;

int button_pin = 5;
int button_state;
int prev_button_state;

int cutoff;

void setup() {

  Serial.begin(9600);

  AudioMemory(10);
  sgtl5000_1.enable();
  sgtl5000_1.volume(0.5);

  pinMode(led_pin1, OUTPUT);
  pinMode(led_pin2, OUTPUT);

  pinMode(button_pin, INPUT_PULLUP);

  waveform1.begin(1, 440, WAVEFORM_SAWTOOTH); //amplitude, frequency, waveform
  pink1.amplitude(1);
  sine1.amplitude(1);

  mixer1.gain(0, .8);
  mixer1.gain(1, 0);
  mixer1.gain(2, 0);

  envelope1.release(150);  // time in milliseconds
  envelope1.noteOn();

  filter1.resonance(.7);  //under .7 is negative, .7 is no resonance, 5 is max

}


void loop()
{

  // read the potentiometers
  pot1_value = analogRead(A1); // analogRead will return a value from 0 to 1023
  pot2_value = analogRead(A2);

  rate1 = pot1_value;
  waveform1.frequency(rate1);
  sine1.frequency(rate1);

  cutoff = map(pot2_value, 0, 1023, 0, 14000);
  filter1.frequency(cutoff);

  analogWrite(led_pin1, pot2_value / 4); //analogwrite is 8 bits (0-255) by defualt while analog read if 10 bits (0-1023)

  prev_button_state = button_state; // remember where the button was at las loop
  button_state = digitalRead(button_pin); // is the button pin high or low

  if (button_state == 0 && prev_button_state == 1) {
    //the button has been pressed
    envelope1.noteOn();
  }

  if (button_state == 1 && prev_button_state == 0) {
    //the button has been released
    envelope1.noteOff();
  }
  
  digitalWrite(led_pin2, !button_state);

  currentMillis = millis();// only need to do this once

  if (currentMillis - previousMillis1 > 200) {
    previousMillis1 = currentMillis;

    Serial.print(" proc: ");
    Serial.print(AudioProcessorUsageMax());
    Serial.print("  mem: ");
    Serial.println(AudioMemoryUsageMax());
    Serial.print(" cutoff: ");
    Serial.println(cutoff);
    Serial.println();

    AudioProcessorUsageMaxReset();
    AudioMemoryUsageMaxReset();

  }


}

