#include <Audio.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <SerialFlash.h>

// GUItool: begin automatically generated code
AudioSynthWaveformSine   sine1;          //xy=133,225
AudioSynthNoisePink      pink1;          //xy=133,267
AudioSynthWaveform       waveform1;      //xy=147,167
AudioFilterStateVariable filter1;        //xy=288,405
AudioMixer4              mixer1;         //xy=297,228
AudioEffectDelay         delay1;         //xy=464,553
AudioMixer4              mixer2;         //xy=465,431
AudioEffectEnvelope      envelope1;      //xy=498,230
AudioOutputI2S           i2s1;           //xy=687,432
AudioConnection          patchCord1(sine1, 0, mixer1, 1);
AudioConnection          patchCord2(pink1, 0, mixer1, 2);
AudioConnection          patchCord3(waveform1, 0, mixer1, 0);
AudioConnection          patchCord4(filter1, 0, mixer2, 0);
AudioConnection          patchCord5(mixer1, envelope1);
AudioConnection          patchCord6(delay1, 0, mixer2, 3);
AudioConnection          patchCord7(mixer2, delay1);
AudioConnection          patchCord8(mixer2, 0, i2s1, 0);
AudioConnection          patchCord9(delay1, 1, i2s1, 1);
AudioConnection          patchCord10(envelope1, 0, filter1, 0);
AudioControlSGTL5000     sgtl5000_1;     //xy=139,53
// GUItool: end automatically generated code


int led_pin1 = 3;    // the number of the first LED pin
int ledState1 = LOW;             // ledState used to set the first LED
long previousMillis1 = 0;        // will store last time LED 1 was updated
long rate1;           // first led rate

int led_pin2 = 4;    // the number of the first LED pin
int ledState2 = LOW;             // ledState used to set the first LED
long previousMillis2 = 0;        // will store last time LED 1 was updated
long rate2;           // first led rate

long previousMillis3 = 0;


long currentMillis;

int pot1_value;
int pot2_value;

int button_pin = 5;
int button_state;
int prev_button_state;

int filter_env_trig;
int filter_env_cnt;

const float chromatic[129] = {
  0, 16.35, 17.32, 18.35, 19.45, 20.6, 21.83, 23.12, 24.5, 25.96, 27.5, 29.14, 30.87, 32.7, 34.65, 36.71, 38.89, 41.2, 43.65, 46.25, 49, 51.91, 55, 58.27, 61.74, 65.41, 69.3, 73.42, 77.78, 82.41, 87.31, 92.5, 98, 103.83, 110, 116.54, 123.47, 130.81, 138.59, 146.83, 155.56, 164.81, 174.61, 185, 196, 207.65, 220, 233.08, 246.94, 261.63, 277.18, 293.66, 311.13, 329.63, 349.23, 369.99, 392, 415.3, 440, 466.16, 493.88, 523.25, 554.37, 587.33, 622.25, 659.25, 698.46, 739.99, 783.99, 830.61, 880, 932.33, 987.77, 1046.5, 1108.73, 1174.66, 1244.51, 1318.51, 1396.91, 1479.98, 1567.98, 1661.22, 1760, 1864.66, 1975.53, 2093, 2217.46, 2349.32, 2489.02, 2637.02, 2793.83, 2959.96, 3135.96, 3322.44, 3520, 3729.31, 3951.07, 4186.01, 4434.92, 4698.63, 4978.03, 5274.04, 5587.65, 5919.91, 6271.93, 6644.88, 7040, 7458.62, 7902.13
};

//http://www.bandnotes.info/tidbits/scales/half-whl.htm
byte natural_minor[7] = {  2, 1, 2, 2, 1, 2, 2}; //natural minor scale
byte major[7] = {2, 2, 1, 2, 2, 2, 1}; //natural minor scale

float arp_bank1[5] = {0, 4, 7, 12}; // major  arpeggio in half steps
float arp_bank2[5] = {0, 3, 6, 9};  //dimminished  arpeggio in half steps

float freq1;
int arp_loc1;

int cutoff;



void setup() {
  Serial.begin(9600);

  //required to setup sudio
  AudioMemory(200); //255 is max
  sgtl5000_1.enable();
  sgtl5000_1.volume(0.5);

  pinMode(led_pin1, OUTPUT);
  pinMode(led_pin2, OUTPUT);

  pinMode(button_pin, INPUT_PULLUP);

  waveform1.begin(1, 440, WAVEFORM_SAWTOOTH);
  sine1.amplitude(1);

  mixer1.gain(0, .8);
  mixer1.gain(1, 0);
  mixer1.gain(2, 0);

  mixer2.gain(0, .6);
  mixer2.gain(3, .4);

  delay1.delay(0, 200);
  delay1.delay(1, 180);

  envelope1.release(150);  // time in milliseconds
  envelope1.noteOn();

  filter1.resonance(3);  //under .7 is negative, .7 is no resonance, 5 is max

}

void loop()
{

  currentMillis = millis();// only need to do this once

  // read the potentiometers
  pot1_value = analogRead(A1); // analogRead will return a value from 0 to 1023
  pot2_value = analogRead(A2);

  rate1 = pot1_value;

  sine1.frequency(rate1);

  cutoff = map(pot2_value, 0, 1023, 0, 14000);
  if (currentMillis - previousMillis3 > 1) {
    previousMillis3 = currentMillis;

    if (filter_env_trig == 1) {
      if (filter_env_cnt < cutoff) {
        filter_env_cnt += 10;
      }

      if (filter_env_cnt > cutoff + 50) {
        filter_env_cnt -= 50;
      }
    }

    if (filter_env_trig == 0) {

      if (filter_env_cnt > 50) {
        filter_env_cnt -= 50;

      }
    }
  }

  filter1.frequency(filter_env_cnt);

  analogWrite(led_pin1, map(filter_env_cnt, 0, 14000, 0, 255));

  prev_button_state = button_state; // remember where the button was at last loop
  button_state = digitalRead(button_pin); // is the button pin high or low

  digitalWrite(led_pin2, !button_state);

  if (button_state == 0 && prev_button_state == 1) {
    //the button has been pressed
    //arp_loc1 = 0;
    previousMillis2 = currentMillis;
    filter_env_trig = 1;
    envelope1.noteOn();
  }

  if (button_state == 1 && prev_button_state == 0) {
    //the button has been released
    filter_env_trig = 0;
    envelope1.noteOff();
  }


  if (currentMillis - previousMillis2 > 135) {
    previousMillis2 = currentMillis;

    arp_loc1 = 60 + random(-12, 12);
    // arp_loc1 = 60 + (random(-3, 3)*4);

    // int r1 = random(7);
    //arp_loc1 = 60 + (major[r1]*(r1+1));

  }

  // freq1 = rate1 * pow(2, (arp_bank1[arp_loc1]) / 12); //https://en.wikipedia.org/wiki/MIDI_tuning_standard
  freq1 = (rate1 / 512.00) * chromatic[arp_loc1]; //https://en.wikipedia.org/wiki/MIDI_tuning_standard
  waveform1.frequency(freq1);



  if (currentMillis - previousMillis1 > 200) {
    previousMillis1 = currentMillis;

    Serial.print(" proc:");
    Serial.print(AudioProcessorUsageMax());
    Serial.print("  mem:");
    Serial.println(AudioMemoryUsageMax());
    Serial.print(" cutoff:");
    Serial.println(filter_env_cnt);
    Serial.println(cutoff);

    AudioProcessorUsageMaxReset();
    AudioMemoryUsageMaxReset();
  }


}

